/**
 * Copyright Copyright 2008-14 Simon Andrews
 *
 *    This file is part of FocalPoint.
 *
 *    FocalPoint is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    FocalPoint is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with StackMeasure; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.FocalPoint.GUI.Thumbnail;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;

import uk.ac.bbsrc.babraham.FocalPoint.GUI.LocationBar;

public class FilePanel extends JPanel implements MouseListener{

	// This is the generic panel which displays information about a file.
	// If that file is an image it will embed within it a thumbnail panel
	// which allows the user to look at each file.
	
	private ThumbnailPanel thumbnail;
	
	protected JPopupMenu popupMenu;

	private File file;
	
	/*
	 * These are the functional subclasses of Thumbnail panel
	 * They will be tried in order
	 * 
	 * It's important that NotAnImagePanel is ALWAYS the LAST
	 * in the set (since it will always succeed, but won't
	 * show a thumbnail).
	 * 
	 * If you want to add a new type then do, but just don't
	 * put it at the end.
	 */
	private static final Class<?> [] panelTypes = new Class [] {
		LSMThumbnailPanel.class,
		AVIThumbnailPanel.class,
		SimpleThumbnailPanel.class,
		BioFormatsThumbnailPanel.class,
		NotAnImageThumbnailPanel.class,
	};
	
	public FilePanel (File file) {
		
		this.file = file;
		addMouseListener(this);
		
		setLayout(new BorderLayout());
		setBorder(BorderFactory.createEmptyBorder(2, 2, 2, 2));
		
		FileDetailsPanel fileDetails = new FileDetailsPanel(file);
		
		add(fileDetails,BorderLayout.CENTER);
		
		thumbnail = getThumbnailView(fileDetails);
		thumbnail.setImageLoadingListener(LocationBar.getInstance());
		fileDetails.setFileSize(thumbnail.getFileSize(fileDetails.file()));
		
		if (thumbnail != null) {
			add(thumbnail,BorderLayout.WEST);
		}
		
		
	}

	public void setIndex (int index) {
		// Have alternating background colours to aid contrast
		if (index%2 == 0) {
			setBackground(Color.LIGHT_GRAY);
		}
		else {
			setBackground(Color.WHITE);
		}
	}
	
	public void cleanUp () {
		/*
		 * Called when we're being destroyed.  Blank stuff to help
		 * the GC along
		 */
		thumbnail.cleanUp();
		thumbnail = null;
		popupMenu = null;
		file = null;
	}
	
	public ThumbnailPanel getThumbnailPanel () {
		return thumbnail;
	}
	
	private ThumbnailPanel getThumbnailView (FileDetailsPanel fileDetails) {

		// There are 3 options for this view.  Either we have a simple
		// image format, a multi image format, or a file which isn't
		// an image.  We need to detect and deal with these differently
				
		ThumbnailPanel tp = null;	
		
		/*
		 * This is a bit complex, but basically we have a list of classes which
		 * subclass Thumbnail panel.  We try to instanciate these via reflection
		 * one at a time, and catch any exception they throw in the constructor.
		 * The relevant exception is IncompatibleFileTypeException, but reflection
		 * can add others so we just catch anything.  As soon as a constructor
		 * succeeds then we use that type to return.
		 * 
		 * The last type we try is the NotAnImage type which should always work.
		 */
		
		for (int i=0;i<panelTypes.length;i++) {
			try {
				tp = (ThumbnailPanel)panelTypes[i].getConstructor(new Class [] {fileDetails.getClass()}).newInstance(new Object [] {fileDetails});
			}
			catch (Exception ifte) {
				continue;
			}
			return tp;
		}

		// If we get here something's gone wrong!
		System.err.println("Couldn't find any type to deal with "+fileDetails.file().getName());
		return tp;
		
	}

	public void mouseClicked(MouseEvent arg0) {}

	public void mouseEntered(MouseEvent e) {
		LocationBar.getInstance().directorySelected(file);
		setBorder(BorderFactory.createLineBorder(Color.RED, 2));
	}

	public void mouseExited(MouseEvent e) {
		setBorder(BorderFactory.createEmptyBorder(2, 2, 2, 2));		
	}

	public void mousePressed(MouseEvent me) {
		
		if (popupMenu == null) {
			popupMenu = new ThumbnailPopupMenu(thumbnail.fileDetails,thumbnail.isImage());
		}
		popupMenu.show(this,me.getX(), me.getY());

		
	}

	public void mouseReleased(MouseEvent e) {
	}
	
	
}
